/*******************************************************************************
 * Dermi: A DHT-based Peer-to-Peer Middleware for Distributed Components.
 * Copyright (C) 2003-2004 Carles Pairot
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ******************************************************************************/
package dermi.activation;

import dermi.*;
import dermi.util.*;
import dermi.listener.*;
import dermi.exception.*;
import dermi.registry.FileRegistry;

import rice.p2p.commonapi.NodeHandle;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Properties;
import java.io.IOException;
import java.io.Serializable;

/**
 * AUTOMATICALLY GENERATED by DERMIC TOOL
 * This class implements the skeleton class for Activator object
 * @author Carles Pairot   <cpairot@etse.urv.es>
 */
public abstract class ActivatorSkel extends EventServer implements RemoteEventListener, Activator {
  String method;
  Hashtable msg = new Hashtable();
  Hashtable subs = new Hashtable();
  private Hashtable receivedEvents = new Hashtable();
  private Hashtable receivedAnycallEvents = new Hashtable();
  // This attribute marks the replica's state as up to date or not
  private boolean replicaState = false;
  private boolean dermiLoadStateResultArrived = false;
  private boolean exception = false;
  private Object dermiLoadStateResult = null;

  /**
   * Constructor: Entry point to the skeleton class
   * @param env Properties Environment properties
   * @throws RemoteException When something goes wrong ;-)
   */
  public ActivatorSkel (Properties env) throws RemoteException {
    super (env);
    className = this.getClass().getSuperclass().getName().replaceAll ("Skel", "");
    uid = dermi.util.UID.getUID();

    // If this is a replica, the UID will come from the environment properties
    if (env.getProperty (Context.CLASS_NAME).equals (className + "Stub")) {
      if (env.getProperty (Context.CLASS_ID) != null) {
        uid = env.getProperty (Context.CLASS_ID);
      }
    }

    subs.put (Context.OBJECTID, uid);
    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Method for obtaining the object's state from any of the available replicas
   */
  public void getReplicaState() throws RemoteException {
    // Wait a certain time for joining to be complete
    try {
      Thread.sleep (2000);
    } catch (Exception e) {}
    // If replica, we must update our status
    try {
      dermiLoadState();
    } catch (NotSatisfiedException ne) {
      // Unable to load replica state: assume it is the first replica
      replicaState = true;
    }
  }

  /**
   * Method invoked each time the skeleton receives an anycall event
   * @param evt Hashtable Information about the received event
   */
  public Hashtable anycallEventArrived (Hashtable evt) throws UnmatchedAnycallMethodException {
    method = (String) evt.get (Context.METHOD);
    try {
      if (method.equals ("dermiGetState")) {
        Serializable res = null;
        Hashtable oldEvent = (Hashtable) receivedAnycallEvents.get (method + (String) evt.get (Context.TIMESTAMP));
        if (oldEvent != null) {
          return oldEvent;
        }
        else {
          // Check for local condition: replica's state must not be stale
          if (replicaState) {
            res = dermiGetState();
            Hashtable msg = new Hashtable();
            msg.put (Context.OBJECTID, uid);
            msg.put (Context.TYPE, Context.DISPATCH);
            msg.put (Context.METHOD, "dermiGetState");
            msg.put (Context.OBJECTCONTENT, res);
            msg.put (Context.REPLICA, new Boolean (true));
            msg.put (Context.ANYCALL_RESULT, new Boolean (false));
            msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
            msg.put (Context.SOURCE_NODEHANDLE, evt.get (Context.SOURCE_NODEHANDLE));
            receivedAnycallEvents.put (method + ((String) evt.get (Context.TIMESTAMP)), msg);
            return (msg);
          }
          else {
            // Local condition not met, message will be routed to another member of the group
            Hashtable msg = new Hashtable();
            msg.put (Context.OBJECTID, uid);
            msg.put (Context.TYPE, Context.DISPATCH);
            msg.put (Context.METHOD, "dermiGetState");
            msg.put (Context.OBJECTCONTENT, evt.get (Context.OBJECTCONTENT));
            msg.put (Context.REPLICA, new Boolean (true));
            msg.put (Context.RESULT, Context.VOID);
            msg.put (Context.ANYCALL_RESULT, new Boolean (true));
            msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
            msg.put (Context.SOURCE_NODEHANDLE, evt.get (Context.SOURCE_NODEHANDLE));
            receivedAnycallEvents.put (method + ((String) evt.get (Context.TIMESTAMP)), msg);
            return (msg);
          }
        }
      }
      else if (method.equals ("activateObject")) {
        Properties res = null;
        Vector args = (Vector) evt.get (Context.OBJECTCONTENT);
        Hashtable oldEvent = (Hashtable) receivedAnycallEvents.get (method + (String) evt.get (Context.TIMESTAMP));
        if (oldEvent != null) {
          return oldEvent;
        }
        else {
          // Check for local condition: replica's state must not be stale
          if (replicaState) {
            res = activateObject ((Properties) args.elementAt (0));
            Hashtable msg = new Hashtable();
            msg.put (Context.OBJ, uid + Context.APPEND_RESULT);
            msg.put (Context.TYPE, Context.DISPATCH);
            msg.put (Context.METHOD, "activateObject");
            msg.put (Context.OBJECTCONTENT, res);
            msg.put (Context.ANYCALL_RESULT, new Boolean (false));
            msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
            msg.put (Context.SOURCE_NODEHANDLE, evt.get (Context.SOURCE_NODEHANDLE));
            receivedAnycallEvents.put (method + ((String) evt.get (Context.TIMESTAMP)), msg);
            return (msg);
          }
          else {
            // Local condition not met, message will be routed to another member of the group
            Hashtable msg = new Hashtable();
            msg.put (Context.OBJ, uid + Context.APPEND_RESULT);
            msg.put (Context.TYPE, Context.DISPATCH);
            msg.put (Context.METHOD, "activateObject");
            msg.put (Context.OBJECTCONTENT, evt.get (Context.OBJECTCONTENT));
            msg.put (Context.RESULT, Context.VOID);
            msg.put (Context.ANYCALL_RESULT, new Boolean (true));
            msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
            msg.put (Context.SOURCE_NODEHANDLE, evt.get (Context.SOURCE_NODEHANDLE));
            receivedAnycallEvents.put (method + ((String) evt.get (Context.TIMESTAMP)), msg);
            return (msg);
          }
        }
      }
    } catch (Exception ex) {
      Hashtable msg = new Hashtable();
      msg.put (Context.OBJ, uid + Context.APPEND_RESULT);
      if (method.equals ("dermiGetState")) {
        msg.put (Context.OBJECTID, uid);
      }
      msg.put (Context.TYPE, Context.EXCEPTION);
      msg.put (Context.METHOD, method);
      if (method.equals ("dermiGetState")) {
        msg.put (Context.REPLICA, new Boolean (true));
      }
      msg.put (Context.OBJECTCONTENT, ex);
      msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
      msg.put (Context.SOURCE_NODEHANDLE, evt.get (Context.SOURCE_NODEHANDLE));
      receivedAnycallEvents.put (method + ((String) evt.get (Context.TIMESTAMP)), msg);
      return msg;
    }
    throw new UnmatchedAnycallMethodException (method);
  }

  /**
   * Method invoked each time the skeleton receives an event
   * @param evt Hashtable Information about the received event
   */
  public void eventArrived (Hashtable evt) {
    method = (String) evt.get (Context.METHOD);

    if (method.equals ("dermiGetState")) {
      dermiLoadStateResult = evt.get (Context.OBJECTCONTENT);
      String type = (String) evt.get (Context.TYPE);
      if (type.equals (Context.EXCEPTION)) {
        exception = true;
      }
      dermiLoadStateResultArrived = true;
      return;
    }
    else if (method.equals("getBytes")) {
      try {
        String aClass = Bus.getClassName (className);
        String fileClass = classStore + aClass + "Stub.class";
        byte data[] = getBytes (fileClass);
        Hashtable msg = new Hashtable();
        msg.put (Context.OBJ, uid + Context.APPEND_RESULT_DYNAMIC);
        msg.put (Context.TYPE, Context.DISPATCH);
        msg.put (Context.METHOD, "getBytes");
        msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
        dispatchEvent (msg, data);
      } catch (IOException ex1) {
        Hashtable msg = new Hashtable();
        msg.put (Context.OBJ, uid + Context.APPEND_RESULT_DYNAMIC);
        msg.put (Context.TYPE, Context.EXCEPTION);
        msg.put (Context.METHOD, "getBytes");
        msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
        try {
          dispatchEvent (msg, ex1);
        } catch (RemoteException e2) {
          e2.printStackTrace();
        }
      } catch (RemoteException ex2) {
        ex2.printStackTrace();
      }
      return;
    }

    if (replicaState) {
      if (method.equals ("addClass")) {
        try {
          Vector args = (Vector) evt.get (Context.OBJECTCONTENT);

          if (receivedEvents.get (method + (String) evt.get (Context.TIMESTAMP)) == null) {
            addClass (((String) args.elementAt (0)), ((ByteWrapper) args.elementAt (1)).getBytes(), ((ByteWrapper) args.elementAt (2)).getBytes(), ((ByteWrapper) args.elementAt (3)).getBytes());
            receivedEvents.put (method + ((String) evt.get (Context.TIMESTAMP)), new Integer(-1));
          }

          Hashtable msg = new Hashtable();
          msg.put (Context.OBJ, uid + Context.APPEND_RESULT);
          msg.put (Context.TYPE, Context.DISPATCH);
          msg.put (Context.METHOD, "addClass");
          msg.put (Context.RESULT, Context.VOID);
          msg.put (Context.SOURCE_NODEHANDLE, (NodeHandle) evt.get (Context.SOURCE_NODEHANDLE));
          msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
          try {
            // Event is dispatched using the event service (response is sent to MANY objects)
            if (evt.get (Context.SOURCE_NODEHANDLE) == null) {
              dispatchEvent (msg);
            }
            else {
              // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
              dispatchDirectEvent (msg);
            }
          } catch (RemoteException e) {
            e.printStackTrace();
          }
        } catch (RemoteException ex) {
          Hashtable msg = new Hashtable();
          msg.put (Context.OBJ, uid + Context.APPEND_RESULT);
          msg.put (Context.TYPE, Context.EXCEPTION);
          msg.put (Context.METHOD, "addClass");
          msg.put (Context.SOURCE_NODEHANDLE, (NodeHandle) evt.get (Context.SOURCE_NODEHANDLE));
          msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
          receivedEvents.put (method + ((String) evt.get (Context.TIMESTAMP)), ex);
          try {
            // Event is dispatched using the event service (response is sent to MANY objects)
            if (evt.get (Context.SOURCE_NODEHANDLE) == null) {
              dispatchEvent (msg, ex);
            }
            else {
              // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
              dispatchDirectEvent (msg, ex);
            }
          } catch (RemoteException e2) {
            e2.printStackTrace();
          }
        }
      }
      else if (method.equals ("activateMultipleObjects")) {
        try {
          Vector args = (Vector) evt.get (Context.OBJECTCONTENT);
          Properties res = null;

          if (receivedEvents.get (method + (String) evt.get (Context.TIMESTAMP)) != null) {
            res = (Properties) receivedEvents.get (method + (String) evt.get (Context.TIMESTAMP));
          }
          else {
            res = activateMultipleObjects ((Properties) args.elementAt (0));
            receivedEvents.put (method + ((String) evt.get (Context.TIMESTAMP)), new Integer(-1));
          }

          Hashtable msg = new Hashtable();
          msg.put (Context.OBJ, uid + Context.APPEND_RESULT);
          msg.put (Context.TYPE, Context.DISPATCH);
          msg.put (Context.METHOD, "activateMultipleObjects");
          msg.put (Context.SOURCE_NODEHANDLE, (NodeHandle) evt.get (Context.SOURCE_NODEHANDLE));
          msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
          try {
            // Event is dispatched using the event service (response is sent to MANY objects)
            if (evt.get (Context.SOURCE_NODEHANDLE) == null) {
              dispatchEvent (msg, (Serializable) res);
            }
            else {
              // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
              dispatchDirectEvent (msg, (Serializable) res);
            }
          } catch (RemoteException e) {
            e.printStackTrace();
          }
        } catch (RemoteException ex) {
          Hashtable msg = new Hashtable();
          msg.put (Context.OBJ, uid + Context.APPEND_RESULT);
          msg.put (Context.TYPE, Context.EXCEPTION);
          msg.put (Context.METHOD, "activateMultipleObjects");
          msg.put (Context.SOURCE_NODEHANDLE, (NodeHandle) evt.get (Context.SOURCE_NODEHANDLE));
          msg.put (Context.TIMESTAMP, (String) evt.get (Context.TIMESTAMP));
          receivedEvents.put (method + ((String) evt.get (Context.TIMESTAMP)), ex);
          try {
            // Event is dispatched using the event service (response is sent to MANY objects)
            if (evt.get (Context.SOURCE_NODEHANDLE) == null) {
              dispatchEvent (msg, ex);
            }
            else {
              // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
              dispatchDirectEvent (msg, ex);
            }
          } catch (RemoteException e2) {
            e2.printStackTrace();
          }
        }
      }
    }
    else {
      System.out.println ("Replica not ready yet.");
    }
  }

  /**
   * This method is used for a not updated replica in order to obtain
   * the object's state
   */
  public ActivatorData loadState() throws RemoteException {
    int timeout = 0;
    // Set timestamp for this event and its possible retransmissions
    String tstamp = "" + System.currentTimeMillis();
    for (int retransmission = 0; retransmission < Context.RETRANSMISSIONS;) {
      Hashtable evt = new Hashtable();
      evt.put (Context.OBJECTID, uid);
      evt.put (Context.TYPE, Context.DISPATCH);
      evt.put (Context.METHOD, "dermiGetState");
      evt.put (Context.TIMESTAMP, tstamp);
      evt.put (Context.SOURCE_NODEHANDLE, conn.getLocalNodeHandle());
      evt.put (Context.OBJECTCONTENT, new Integer (-1));
      dermiLoadStateResultArrived = false;
      try {
        conn.anycall (evt);
      }
      catch (SessionException ex1) {
        throw new RemoteException (ex1.getMessage());
      }
      timeout = 0;
      while (!dermiLoadStateResultArrived && timeout < Context.TIMEOUT) {
        try {
          Thread.currentThread().sleep (Context.DELAY);
        } catch (InterruptedException ex) {
          ex.printStackTrace();
        }
        timeout++;
      }
      if (exception) {
        exception = false;
        RemoteException ex = (RemoteException) dermiLoadStateResult;
        throw ex;
      } else if (timeout == Context.TIMEOUT) {
        retransmission++;
        continue;
      }
      break;
    }
    if (timeout == Context.TIMEOUT) {
      throw new TimeoutException ("Object seems to be down.");
    }
    else {
      // Replica's state now up to date
      replicaState = true;
      System.out.println ("Object's state restored from replica.");
      return (ActivatorData) dermiLoadStateResult;
    }
  }

  public abstract ActivatorData dermiGetState();
  public abstract void dermiLoadState() throws RemoteException;

  /**
   * Closes the skeleton
   * @throws RemoteException If something goes wrong ;-)
   */
  public void close() throws RemoteException {
    try {
      conn.unsubscribe (this, subs);
      dermi.util.Bus.close();
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Initializes the skeleton
   * @throws RemoteException If something goes wrong ;-)
   */
  public void init() throws RemoteException {
    conn = dermi.util.Bus.getBus (eref);
    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Returns a stub copy of this class
   * @throws RemoteException If something goes wrong ;-)
   * @return ERemote The returned stub
   */
  public ERemote copy() throws RemoteException {
    ERemote ref = FileRegistry.loadStub (getERef());
    return (ERemote) ref;
  }
}
